<?php // $Id: pay_form.inc,v 1.21 2010/08/27 10:34:46 jerdavis Exp $

/**
 * @file
 * The base class for payment activities.  All payment form classes should
 * extend this.
 */

class pay_form extends pay {

  var $pfid;
  var $title;
  var $status = 1;
  var $embeddable = FALSE;
  var $uid;
  var $min_amount = 1.00;
  var $max_amount = 1000.00;
  var $menu_path;
  var $total_goal;
  var $currency;
  var $pay_methods = array();
  var $notes_title = 'Comments';
  var $notes_format = FILTER_FORMAT_DEFAULT;
  var $notes_description = '';

  var $table = 'pay_form';
  var $key = 'pfid';

  function title() {
    return check_plain($this->title);
  }

  function set_transaction($values) {
    module_load_include('inc', 'pay', 'includes/handlers/pay_transaction');
    $this->transaction = New pay_transaction($values);
    if (!$this->transaction->pfid) $this->transaction->pfid = $this->pfid;
    $this->transaction->save($values);
    return $this->transaction;
  }

  function pay_method_list() {
    $list = array();
    $res = db_query("SELECT * FROM {pay_method} WHERE STATUS = 1 ORDER BY title");
    while ($row = db_fetch_object($res)) {
      $list[$row->pmid] = $row->title;
    }
    return $list;
  }

  function set_embeddable($value = FALSE) {
    $this->embeddable = (bool) $value;
  }

  function embeddable() {
    return (bool) $this->embeddable;
  }

  function settings_form(&$form, &$form_state) {
    parent::settings_form($form, $form_state);
    $group = $this->handler();

    // If the parent form has not supplied a title, add one.
    if (!isset($form['title'])) {
      $form[$group]['title'] = array(
        '#type' => 'textfield',
        '#title' => t('Title'),
        '#default_value' => $this->title,
        '#required' => TRUE,
        '#description' => t('This title will appear as the title of the form page and in listings.')
      );
    }
    
    $form[$group]['min_amount'] = array(
      '#type' => 'textfield',
      '#size' => 10,
      '#title' => t('Minimum amount'),
      '#description' => t('The minimum allowed payment on this form. If there are service fees or other costs associated with receiving payments, you will want this to set this amount higher than those costs.'),
      '#default_value' => $this->min_amount,
    );

    $form[$group]['max_amount'] = array(
      '#type' => 'textfield',
      '#size' => 10,
      '#title' => t('Maximum amount'),
      '#description' => t('The maximum allowed amount for this form.  If your payment method has a limit, this setting should reflect it.'),
      '#default_value' => $this->max_amount,
    );

    $form[$group]['pay_methods'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Available payment methods'),
      '#description' => t('Payment methods available for this form.'),
      '#options' => $this->pay_method_list(),
      '#default_value' => $this->pay_methods,
    );
  }

  function pay_methods() {
    $methods = array();
    foreach ($this->pay_methods as $pmid => $status) {
      if ($status) $methods[$pmid] = pay_method_load($pmid);
    }
    return $methods;
  }

  function pay_method_form(&$form_state) {
    $form = array('#tree' => TRUE);
    $methods = $this->pay_methods();
    $options = array();
    foreach ($methods as $pmid => $pay_method) $options[$pmid] = $pay_method->title;

    $form['selected'] = array(
      '#type' => 'radios',
      '#options' => $options,
      '#required' => TRUE,
      '#access' => count($methods) > 1,
      '#default_value' => count($methods) == 1 ? $methods : NULL,
    );

    foreach ($methods as $pay_method) {
      if ($pay_method->access('default')) {
        $form[$pay_method->pmid] = $pay_method->form($form_state);
        $form[$pay_method->pmid]['pmid'] = array(
          '#type' => 'value', '#value' => $pay_method->pmid,
        );
      }
    }
    return $form;
  }

  function form(&$form, &$form_state) {
    parent::form($form, $form_state);

    $group = $this->handler();

    $form[$group]['total'] = array(
      '#type' => 'textfield',
      '#size' => 6,
      '#title' => t('Amount'),
    );

    $form[$group]['pay_method'] = $this->pay_method_form($form_state);

    $form[$group]['notes'] = array(
      '#type' => 'textarea',
      '#title' => $this->notes_title,
      '#description' => $this->notes_description,
    );
  }

  function form_alter(&$form, &$form_state) {
    $group = $this->handler();

    // Special handling for "amount-only" forms.
    if ($form_state['pay_form_type'] == 'amount') {
      $visible = array();

      foreach (element_children($form[$group]) as $key) {
        if (!in_array($key, array('amount', 'amount_other', 'total'))) {
          $form[$group][$key]['#access'] = FALSE;
        }

        // Keep track of any visible elements.
        if ($form[$group][$key]['#access'] !== FALSE) $visible[] = $key;

        // Change the 'action' to the ultimate destination for the form.
        $form['#action'] = $this->menu_path();

        // Change the 'method' to GET ...?
        $form['#method'] = 'get';
      }
    }

    // If only one field is visible, hide its label.
    if (count($visible == 1)) {
      unset($form[$group][$visible[0]]['#title']);
    }
  }

  function form_validate($form, &$form_state) {
    parent::form_validate($form, $form_state);
    $values = $this->form_values($form_state);
    
    // Remove non-numeric characters from the total. 
    $total = (float) preg_replace('/[^\d\.]/', '', $values['total']);

    // Confirm that the amount falls within our min/max settings.
    if (($total < $this->min_amount) || ($total > $this->max_amount) ) {
      $error = t('Please enter an amount between %min and %max', array('%min' =>$this->min_amount, '%max' => $this->max_amount));
      form_set_error('total', $error);
    }

    // If there is no total value per payment method, set the global total.
    foreach ($values['pay_method']['selected'] as $pmid => $status) {
      if (!$status) continue;
      $key = $this->handler();
      $method_values = $form_state['values'][$key]['pay_method'][$pmid];
      $method_element = $form[$key]['pay_method'][$pmid];
      if (!$method_values['total']) {
        form_set_value($method_element['total'], (float)$total, $form_state);
        $method_values['total'] = $total;
      }
      $pay_method = pay_method_load($method_values);
      $pay_method->pay_method_validate($form, $form_state, $method_element);
    }
  }

  function form_submit($form, &$form_state) {
    parent::form_submit($form, $form_state);
    $values = $this->form_values($form_state);

    // Add a new transaction for this form with status as default of 'pending'.
    $transaction = $this->set_transaction($values);

    // Run payment activities for the 1 or more selected payment methods.
    $selected = $values['pay_method']['selected'];
    if (!is_array($selected)) $selected = array($selected => 1);

    foreach ($selected as $pmid => $status) {
      if (!$status) continue;
      $pay_method_values = $values['pay_method'][$pmid];
      $pay_method['pmid'] = $pmid;
      $pay_method = pay_method_load($pay_method_values);
      $transaction->save($pay_method_values);

      $activity = $transaction->add_activity($pay_method, $pay_method->pay_form_activity);
      $activity->do_activity();
      $transaction->update_status($activity->timestamp);
    }

    // Trigger a special hook if there's a goal amount and we have reached it.
    if ($this->total_goal() && ($this->total_paid() >= $this->total_goal())) {
      $this->drupal_invoke('pay_form_goal', $form_state);
    }

    unset($form_state['rebuild'], $form_state['storage']);
  }

  function set_menu_path($val = NULL) {
    $this->menu_path = $val;
  }

  function menu_path() {
    return check_plain($this->menu_path);
  }

  function total() {
    return (float) db_result(db_query("SELECT SUM(total) as total FROM {pay_transaction} WHERE pfid = %d", $this->pfid));
  }

  function total_paid() {
    return (float) db_result(db_query("SELECT SUM(total_paid) as total FROM {pay_transaction} WHERE pfid = %d", $this->pfid));
  }
  
  function total_goal() {
    return (float) $this->total_goal;
  }

  function transaction_count() {
    return (int) db_result(db_query("SELECT COUNT(*) FROM {pay_transaction} WHERE pfid = %d", $this->pfid));
  }
  
  function currency() {
    return db_result(db_query("SELECT currency FROM {pay_form} WHERE pfid=%d", $this->pfid));
  }
}
